;    Original version by Flavio Matsumoto
;    Improvements by Einar Saukas
;    Includes delay routine by Jan Bobrowski

;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.

;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.

;    You should have received a copy of the GNU General Public License
;    along with this program.  If not, see <http://www.gnu.org/licenses/>.

        ORG 32768
VTMain:
        ; Load delay value
        LD DE,5000

        ; Setup an "empty" IM2 routine
        LD HL,$FFFE
        LD ($FEFF),HL       ; interruption will execute code at $FFFE
        LD (HL),$FB         ; opcode for "EI" (4T)
        INC HL
        LD (HL),$C9         ; opcode for "RET" (10T)

        ; Enable IM 2
        DI
        LD A,$FE
        LD I,A
        IM 2
        EI

VTStart:
        ; Check for pressed key
        LD BC,254           ; B signals to read all keys, C is port address.
        IN A,(C)            ; Read keyboard.
        CPL                 ; If some key was pressed,
        AND %011111
        JR NZ,VTExit        ; then exit

        ; Change border some time after next interrupt
        LD A,2              ; Value for red border
        HALT                ; Wait for the maskable interrupt (0 to 3T)
        OUT (C),A           ; Change border to red (12T)
        LD B,D              ; Copy delay value to BC (4T)
        LD C,E              ; Copy delay value to BC (4T)
        CALL DELAY          ; configurable delay
        LD BC,254           ; port address (10T)
        LD A,5              ; Value for cyan border (7T)
        OUT (C),A           ; Change border color. (12T)

        ; "Force" a variable delay before the next HALT
        LD A,R
        AND 15
        LD B,A
VTLoop:
        DJNZ VTLoop

        ; Next frame
        JR VTStart

        ; Disable IM 2
VTExit:
        DI
        LD A,$3F
        LD I,A
        IM 1
        EI
        RET

#include "delay.asm"
